@@ -42,6 +42,20 @@ class Worker:
             hdlr=logging.NullHandler(),
         )
 
+        signal.signal(signal.SIGUSR1, self.stop_signal_handler)
+
+    def stop_signal_handler(
+        self,
+        signal_num: int,
+        frame: typing.Optional[types.FrameType],
+    ) -> None:
+        self.logger.info(
+            msg='stop signal has been received',
+        )
+        signal.signal(signal.SIGUSR1, signal.SIG_DFL)
+
+        raise WorkerStop()
+
     def init_worker(
         self,
     ) -> None:
@@ -89,9 +103,9 @@ class Worker:
         )
 
         connector_obj: connector.Connector
-        if self.config.connector.type == connector.mongo.Connector.name:
+        if self.config.connector.type == 'mongo':
             connector_obj = connector.mongo.Connector(**self.config.connector.params)
-        elif self.config.connector.type == connector.redis.Connector.name:
+        elif self.config.connector.type == 'redis':
             connector_obj = connector.redis.Connector(**self.config.connector.params)
         else:
             raise ValueError(f'connector type {self.config.connector.type} is not supported')
@@ -115,6 +129,15 @@ class Worker:
                 number_of_threads=self.config.number_of_threads,
             )
 
+    def get_trace_id(
+        self,
+    ) -> typing.Optional[str]:
+        current_task = self.executor_obj.get_current_task()
+        if current_task:
+            return current_task.trace_id
+        else:
+            return None
+
     def purge_tasks(
         self,
         task_name: typing.Optional[str] = None,
@@ -139,9 +162,11 @@ class Worker:
         task_name: typing.Optional[str] = None,
         priority: str = 'NORMAL',
         consumable_from: int = 0,
+        trace_id: typing.Optional[str] = None,
     ) -> bool:
         task = objects.Task(
             kwargs=kwargs,
+            trace_id=trace_id if trace_id is not None else self.get_trace_id(),
         )
 
         return self.broker.push_task(
@@ -157,10 +182,13 @@ class Worker:
         task_name: typing.Optional[str] = None,
         priority: str = 'NORMAL',
         consumable_from: int = 0,
+        trace_id: typing.Optional[str] = None,
     ) -> bool:
+        trace_id = trace_id if trace_id is not None else self.get_trace_id()
         tasks = [
             objects.Task(
                 kwargs=kwargs,
+                trace_id=trace_id,
             )
             for kwargs in kwargs_list
         ]
@@ -193,31 +221,11 @@ class Worker:
     def iterate_tasks(
         self,
     ) -> typing.Iterable[objects.Task]:
-        stop_signal_received = False
-
-        def stop_signal_handler(
-            signal_num: int,
-            frame: types.FrameType,
-        ) -> None:
-            nonlocal stop_signal_received
-
-            stop_signal_received = True
-
-            self.logger.info(
-                msg='stop signal has been received',
-            )
-            signal.signal(signal.SIGUSR1, signal.SIG_DFL)
-
-        signal.signal(signal.SIGUSR1, stop_signal_handler)
-
         time_with_no_tasks = 0
         run_forever = self.config.max_tasks_per_run == 0
         tasks_left = self.config.max_tasks_per_run
 
         while tasks_left > 0 or run_forever:
-            if stop_signal_received:
-                break
-
             if run_forever:
                 number_of_tasks_to_pull = self.config.tasks_per_transaction
             else:
@@ -246,9 +254,6 @@ class Worker:
                         iterated_tasks += 1
 
                         yield task
-
-                        if stop_signal_received:
-                            break
                 finally:
                     if iterated_tasks < len(tasks):
                         self.push_tasks(
@@ -411,8 +416,8 @@ class Worker:
                 task=task,
                 returned_value=returned_value,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_success handler has failed: {exception}',
@@ -451,8 +456,8 @@ class Worker:
             self.handle_requeue(
                 task=task,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_failure handler has failed: {exception}',
@@ -489,8 +494,8 @@ class Worker:
             self.handle_requeue(
                 task=task,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_timeout handler has failed: {exception}',
@@ -515,8 +520,8 @@ class Worker:
             self.on_retry(
                 task=task,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_retry handler has failed: {exception}',
@@ -541,8 +546,8 @@ class Worker:
             self.on_max_retries(
                 task=task,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_max_retries handler has failed: {exception}',
@@ -567,8 +572,8 @@ class Worker:
             self.on_requeue(
                 task=task,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_requeue handler has failed: {exception}',
@@ -593,8 +598,8 @@ class Worker:
             self.on_starvation(
                 time_with_no_tasks=time_with_no_tasks,
             )
-        except WorkerInterrupt as exception:
-            raise exception
+        except WorkerInterrupt:
+            raise
         except Exception as exception:
             self.logger.error(
                 msg=f'on_starvation handler has failed: {exception}',
